
#include "HL_Switch.h"
/*
   (c)2014-2017 Forward Computing and Control Pty. Ltd.
   NSW Australia, www.forward.com.au
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This generated code may be freely used for both private and commercial use
   provided this copyright is maintained.
*/

HL_Switch::HL_Switch(char _cmd, pfodDwgs *_dwgsPtr) : pfodControl(_dwgsPtr) {
  buttonCmd = _cmd;
  z_idx = pfodDwgs::reserveIdx(3);
  baseCol = 0; baseRow = 0; baseScaling = 1.0f;
  value = 0; // initially input
  forceUpdate = false;
  saveCurrentState();
}

void HL_Switch::setBase(int c, int r) {
  baseCol = c;
  baseRow = r;
}

void HL_Switch::setBaseScaling(double _scaling) {
  baseScaling = (float)_scaling;
}

void HL_Switch::forceFullUpdate() {
  forceUpdate = true;
}


char HL_Switch::getCmd() {
  return buttonCmd;
}


void HL_Switch::setValue(int newValue) {
  int limit = 1;
  if (newValue < 0) {
    newValue = 0;
  } else if (newValue > limit) {
    newValue = limit;
  }
  pfodControl::setValue(newValue);
}

void HL_Switch::draw() {
  dwgsPtr->pushZero(baseCol, baseRow, baseScaling);
  dwgsPtr->rectangle().centered().color(dwgsPtr->GREY).rounded().filled().size(12, 3).send();
  // add on/off labels with higher z-order then the button rectangle so they are always drawn on top of it
  dwgsPtr->label().fontSize(-2).bold().text(F("High")).idx(z_idx + 1).color(dwgsPtr->WHITE).offset(-3, 0).send();
  dwgsPtr->label().fontSize(-2).bold().text(F("Low")).idx(z_idx + 2).color(dwgsPtr->WHITE).offset(3, 0).send();
  fullUpdate(); // update with current state
  dwgsPtr->popZero(); // pop Base push and scale
}

void HL_Switch::fullUpdate() {
  dwgsPtr->rectangle().filled().rounded().idx(z_idx).size(6, 4).offset(((value != 0) ? -6 : 0), -2)
  .color((value != 0) ? dwgsPtr->RED : dwgsPtr->BLACK).send();
  // update the touchZone and action to the opposite side then is currently set
  dwgsPtr->touchZone().cmd(buttonCmd).size(6, 4).offset(((value != 0) ? 0 : -6), -2).filter(dwgsPtr->TOUCH)
  .send();
  dwgsPtr->touchAction().cmd(buttonCmd).action( // create the action
    dwgsPtr->rectangle().filled().rounded().idx(z_idx).size(6, 3).offset(((value != 0) ? 0 : -6), -1.5)
    .color((value != 0) ? dwgsPtr->BLACK : dwgsPtr->RED ) //
  ).send(); // send the touchAction
}

boolean HL_Switch::haveStateChange() {
  return ((value != previousValue) || forceUpdate);
}

void HL_Switch::saveCurrentState() {
  previousValue = value;
  forceUpdate = false;
}

void HL_Switch::update() {
  if (haveStateChange()) {
    fullUpdate();
  }
  saveCurrentState();
}


